from textual.widgets import DataTable, Static
from textual.app import ComposeResult
from textual.containers import Container
import asyncio
import financedatabase as fd

class StockMarketTab(Container):
    """MarketTab for displaying stock market data in a 5x2 grid layout asynchronously."""

    REFRESH_INTERVAL = 60  # Update every 60 seconds

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.market_tables = {}  # Store references to each DataTable
        self.stock_list = self.fetch_stock_list()  # Fetch stocks once and reuse

    def fetch_stock_list(self):
        """Fetch 100 stocks from FinanceDatabase and split into 10 groups."""
        database = fd.show_options()
        stocks = fd.select_equities(country="Indonesia", limit=100)  # Fetch 100 stocks

        stock_items = list(stocks.items())  # Convert dictionary to list of (symbol, details)
        grouped_stocks = [stock_items[i:i + 10] for i in range(0, len(stock_items), 10)]  # Split into 10 groups

        return grouped_stocks  # Returns a list of 10 groups, each containing 10 stocks

    def compose(self) -> ComposeResult:
        """Create a 5x2 grid layout with 10 sections for stock market data."""
        with Container(id="market-grid"):
            for i in range(10):  # Create 10 tables
                table = DataTable(id=f"table-{i+1}", classes="market-table")
                self.market_tables[f"Table {i+1}"] = table

                # Add section headers with tables in grid cells
                yield Container(
                    Static(f"Stock Table {i+1}", classes="section-header"),
                    table,
                    id=f"grid-cell-{i}",
                    classes="grid-cell"
                )

    async def on_mount(self) -> None:
        """Initialize tables and start data updates asynchronously."""
        self.setup_tables()
        asyncio.create_task(self.refresh_data_periodically())  # Run background updates

    def setup_tables(self) -> None:
        """Set up column headers for all stock market tables."""
        for table in self.market_tables.values():
            table.clear()
            if not table.columns:
                table.add_column("Stock", width=18)
                table.add_column("Symbol", width=10)
                table.add_column("Price", width=12)
                table.add_column("Change (1D)", width=12)
                table.add_column("Change% (1D)", width=12)
                table.add_column("Change% (7D)", width=12)
                table.add_column("Change% (30D)", width=12)

    async def fetch_single_stock(self, symbol: str):
        """Fetch price & percentage changes asynchronously with threading for non-blocking execution."""
        import yfinance as yf

        try:
            hist = await asyncio.to_thread(lambda: yf.Ticker(symbol).history(period="3mo"))

            if hist.empty:
                return symbol, None, None, None, None, None, None

            latest_price = hist["Close"].iloc[-1]  # Last closing price
            prev_1d = hist["Close"].iloc[-2] if len(hist) > 1 else latest_price
            prev_7d = hist["Close"].iloc[-7] if len(hist) > 7 else latest_price
            prev_30d = hist["Close"].iloc[-30] if len(hist) > 30 else latest_price

            return symbol, latest_price, latest_price - prev_1d, \
                   (latest_price - prev_1d) / prev_1d * 100 if prev_1d else 0, \
                   (latest_price - prev_7d) / prev_7d * 100 if prev_7d else 0, \
                   (latest_price - prev_30d) / prev_30d * 100 if prev_30d else 0

        except Exception as e:
            print(f"⚠️ Error fetching {symbol}: {e}")
            return symbol, None, None, None, None, None, None

    async def fetch_stock_data(self):
        """Fetch market data for 100 stocks concurrently."""
        stock_symbols = [symbol for group in self.stock_list for symbol, _ in group]  # Extract stock symbols
        tasks = [self.fetch_single_stock(symbol) for symbol in stock_symbols]
        return await asyncio.gather(*tasks)

    async def refresh_data_periodically(self):
        """Continuously refresh data without freezing the UI."""
        while True:
            await self.refresh_data()
            await asyncio.sleep(self.REFRESH_INTERVAL)  # Prevents UI blocking

    async def refresh_data(self) -> None:
        """Fetch new data and update tables asynchronously."""
        rows = await self.fetch_stock_data()
        stocks_per_table = 10

        for table_index, table_name in enumerate(self.market_tables.keys()):
            table = self.market_tables[table_name]
            table.clear(columns=False)  # Clear rows but retain columns

            for row_index in range(stocks_per_table):
                stock_index = table_index * stocks_per_table + row_index
                if stock_index >= len(rows):
                    continue

                symbol, price, change, change_percent_1d, change_percent_7d, change_percent_30d = rows[stock_index]

                # Format values for display
                price_str = f"{price:,.2f}" if price is not None else "-"
                change_str = f"{change:+,.2f}" if change is not None else "-"
                change_percent_1d_str = f"{change_percent_1d:+,.2f}%" if change_percent_1d is not None else "-"
                change_percent_7d_str = f"{change_percent_7d:+,.2f}%" if change_percent_7d is not None else "-"
                change_percent_30d_str = f"{change_percent_30d:+,.2f}%" if change_percent_30d is not None else "-"

                color_change = "green" if change and change > 0 else "red"
                color_percent_1d = "green" if change_percent_1d and change_percent_1d > 0 else "red"
                color_percent_7d = "green" if change_percent_7d and change_percent_7d > 0 else "red"
                color_percent_30d = "green" if change_percent_30d and change_percent_30d > 0 else "red"

                # Add row with color formatting
                table.add_row(
                    f"Stock {stock_index+1}",
                    symbol,
                    price_str,
                    f"[{color_change}]{change_str}[/]",
                    f"[{color_percent_1d}]{change_percent_1d_str}[/]",
                    f"[{color_percent_7d}]{change_percent_7d_str}[/]",
                    f"[{color_percent_30d}]{change_percent_30d_str}[/]",
                )

